CREATE OR REPLACE PACKAGE BODY
---------------------------------------------------------------------------
--
-- Licensed Materials - Property of IBM
-- 5724-T05,5724-T06
--
-- (C) Copyright IBM Corp. 2003,2008. All Rights Reserved.
--
-- US Government Users Restricted Rights - Use, duplication or
-- disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
--
---------------------------------------------------------------------------
-- (#) ./pweb/vendor/EnterpriseTrafficCache/pe_performance_monitor.sql BL_core_schema_2.2.5.2.02_20080609  192007
-- (#) 
--
-- Web Performance Package that gathers  JVM specific and report statistics.
--
-- Error Codes:
-- -20xxx
-- ------------------------------------------------------------------------
pe_perf_stats IS

  -- ----------------------------------------------------------------------
  -- Private helper method to get the queue count
  -- ----------------------------------------------------------------------
  FUNCTION getQueueCount(pAgentType IN pe_agent_types.agent_type_name%TYPE)
    RETURN pe_perf_stats_general.scheduled_report_queue_count%TYPE
  IS
    vQueueCount pe_perf_stats_general.scheduled_report_queue_count%TYPE := 0;
  BEGIN
     SELECT COUNT(a.activity_id) 
       INTO vQueueCount
       FROM pe_activity a, pe_agent_types t
      WHERE a.agent_type_id = t.agent_type_id
        AND a.state = 'RUNNABLE'
        AND upper(t.agent_type_name) = upper(pAgentType);

     RETURN vQueueCount;
  END getQueueCount;

  -- ----------------------------------------------------------------------
  -- Private helper method to update the general stats
  -- ----------------------------------------------------------------------
  PROCEDURE updateGeneralStats(pStartTime IN DATE,
                               pEndTime   IN DATE,
                               pActiveUsersCount  IN pe_perf_stats_general.active_user_count%TYPE,
	      		               pMaxMemory         IN pe_perf_stats_general.max_memory%TYPE,
	                           pAvailableMemory   IN pe_perf_stats_general.available_memory%TYPE)
  IS
    vReportStartedCount pe_perf_stats_general.report_started_count%TYPE := 0;
    vReportCompletedCount pe_perf_stats_general.report_completed_count%TYPE := 0;
    vScheduledReportQueueCount pe_perf_stats_general.scheduled_report_queue_count%TYPE := 0;
    vImmediateReportQueueCount pe_perf_stats_general.immediate_report_queue_count%TYPE := 0;
  BEGIN
    --
    --Set schedule queue count
    vScheduledReportQueueCount := getQueueCount('SCHEDULED_REPORT');
    vImmediateReportQueueCount := getQueueCount('IMMEDIATE_REPORT');

    SELECT count(*) 
      INTO vReportStartedCount
      FROM task_status ts, active_task at, active_job aj  
     WHERE at.task_id = ts.task_id
       AND at.job_id = ts.job_id
       AND at.job_id = aj.job_id
       AND aj.job_type in (1,10)
       AND nvl(ts.started, ts.ended) >= pStartTime
       AND nvl(ts.started, ts.ended) < pEndTime;

    SELECT count(*) 
      INTO vReportCompletedCount
      FROM pe_perf_stats_report sr  
     WHERE sr.end_time >= pStartTime
       AND sr.end_time < pEndTime;

    MERGE INTO pe_perf_stats_general g
    USING (SELECT 1 FROM dual) d
    ON (g.tstamp = pEndTime)
    WHEN MATCHED THEN
      UPDATE
         SET g.max_memory                   = pMaxMemory,
             g.active_user_count            = pActiveUsersCount,
             g.available_memory             = pAvailableMemory,
             g.report_started_count         = vReportStartedCount,
             g.report_completed_count       = vReportCompletedCount,
             g.scheduled_report_queue_count = vScheduledReportQueueCount,
             g.immediate_report_queue_count  = vImmediateReportQueueCount
    WHEN NOT MATCHED THEN
      INSERT
        (tstamp,
         active_user_count,
         max_memory,
         available_memory,
         report_started_count,
         report_completed_count,
         scheduled_report_queue_count,
         immediate_report_queue_count)
      VALUES
        (pEndTime,
         pActiveUsersCount,
         pMaxMemory,
         pAvailableMemory,
         vReportStartedCount,
         vReportCompletedCount,
         vScheduledReportQueueCount,
         vImmediateReportQueueCount);

  END updateGeneralStats;

  -- ----------------------------------------------------------------------
  -- Private helper method to update the report queue stats
  -- ----------------------------------------------------------------------
  PROCEDURE updateReportQueueStats(pStartTime IN DATE,
                                   pEndTime   IN DATE)
  IS
  BEGIN

    MERGE INTO pe_perf_stats_report_queue q
    USING (SELECT l.agent_type_id,
                  l.datasource_id,
                  AVG(nvl(l.wait_len_time /
                          decode(l.wait_time, 0, NULL, l.wait_time),
                          0)) avg_queue_length,
                  MAX(nvl(l.wait_len_time /
                          decode(l.wait_time, 0, NULL, l.wait_time),
                          0)) max_queue_length,
                  MIN(nvl(l.wait_len_time /
                          decode(l.wait_time, 0, NULL, l.wait_time),
                          0)) min_queue_length
             FROM pe_agent_load l, pe_agent_types t
            WHERE l.agent_type_id = t.agent_type_id
              AND t.agent_type_name IN ('IMMEDIATE_REPORT', 'SCHEDULED_REPORT')
              AND tstamp BETWEEN pStartTime AND pEndTime
            GROUP BY l.agent_type_id, l.datasource_id) x
    ON (q.tstamp = pEndTime AND q.agent_type_id = x.agent_type_id AND q.datasource_id = x.datasource_id)
    WHEN MATCHED THEN
      UPDATE
         SET q.avg_queue_length = x.avg_queue_length,
             q.max_queue_length = x.max_queue_length,
             q.min_queue_length = x.min_queue_length
    WHEN NOT MATCHED THEN
      INSERT
        (tstamp,
         agent_type_id,
         datasource_id,
         avg_queue_length,
         max_queue_length,
         min_queue_length)
      VALUES
        (pEndTime,
         x.agent_type_id,
         x.datasource_id,
         x.avg_queue_length,
         x.max_queue_length,
         x.min_queue_length);

  END updateReportQueueStats;

  -- ------------------------------------------------------------------------
  -- Persist pweb performance statistics about the general pweb environment
  -- and the pweb queue environment.
  -- ------------------------------------------------------------------------
  PROCEDURE addStatistics(pActiveUsersCount  IN pe_perf_stats_general.active_user_count%TYPE,
	      		          pMaxMemory         IN pe_perf_stats_general.max_memory%TYPE,
	                      pAvailableMemory   IN pe_perf_stats_general.available_memory%TYPE)
  IS
    vStartTime DATE;
    vEndTime DATE;
    vSysDate DATE;

  BEGIN
 
    -- Compute start and end time of previous half hour
    vSysDate := sysdate;
    SELECT currdate + mod(trunc(currdate+(1/24),'HH24') - currdate,1/48) 
      INTO vEndTime
      FROM (SELECT vSysDate + 1/86400 currdate
              FROM dual);
    vEndTime := vEndTime - 30 / (24 * 60); -- Subtract 30 minutes in day units.
    vStartTime := vEndTime - 30 / (24 * 60); -- Subtract 30 minutes in day units.

    updateGeneralStats(vStartTime, vEndTime, pActiveUsersCount, pMaxMemory, pAvailableMemory);

    updateReportQueueStats(vStartTime, vEndTime);

  END addStatistics;
  
  -- ------------------------------------------------------------------------
  -- Persist information about the report statistics
  -- ------------------------------------------------------------------------
  PROCEDURE addReportDBStats(pTaskId    IN   pe_perf_report_db_stats.task_id%TYPE)
  IS
    vStatId pe_perf_db_stat_name.stat_id%TYPE;
  BEGIN
     INSERT INTO pe_perf_db_stat_name
       (stat_id, stat_name)
       (SELECT pe_seq_perf_db_stat.NEXTVAL, stat_name
          FROM pe_perf_report_db_stats_tmp t
         WHERE NOT EXISTS (SELECT 1 
                             FROM pe_perf_db_stat_name n
                            WHERE UPPER(n.stat_name) = UPPER(t.stat_name)));

    INSERT INTO pe_perf_report_db_stats
       (tstamp, task_id, stat_id, stat_value)
       (SELECT sysdate, pTaskId, n.stat_id, t.stat_value
          FROM pe_perf_report_db_stats_tmp t, pe_perf_db_stat_name n
         WHERE UPPER(n.stat_name) = UPPER(t.stat_name));

    -- First value in wins
    EXCEPTION
     WHEN DUP_VAL_ON_INDEX THEN
       NULL;

  END addReportDBStats;

  -- ------------------------------------------------------------------------
  -- Persist information about the report run
  -- ------------------------------------------------------------------------
  PROCEDURE addReportStats(pTaskId      IN pe_perf_stats_report.task_id%TYPE,
                           pReportAttrs IN pe_perf_stats_report.report_attrs%TYPE,
                           pTableCount  IN pe_perf_stats_report.table_count%TYPE,
                           pColumnCount IN pe_perf_stats_report.column_count%TYPE)
  IS
    vE2EDataSourceId pe_datasource.datasource_id%TYPE;
  BEGIN

    vE2EDataSourceId := pe_datasource_api.getEnterpriseDataSourceId();

    INSERT INTO pe_perf_stats_report
      (tstamp,
       task_id,
       report_id,
       datasource_id,
       user_id,
       queue_time,
       start_time,
       end_time,
       job_type,
       report_attrs,
       status_code,
       table_count,
       column_count)
      (SELECT sysdate,
              ta.task_id task_id,
              r.id,
              nvl(a.datasource_id, vE2EDataSourceId) datasourceId,
              r.userid,
              a.active_time queue_time,
              ts.started start_time,
              ts.ended end_time,
              ta.job_type job_type,
              pReportAttrs,
              ts.status_code,
              pTableCount,
              pColumnCount
         FROM task_status          ts,
              pe_task_activity     ta,
              active_job           aj,
              pe_activity          a,
              rg_tasklist          tl,
              pm_report            r
        WHERE ta.task_id = pTaskId
          AND ts.task_id = ta.task_id
          AND ta.activity_id = a.activity_id
          AND ta.job_id = aj.job_id
          AND tl.id = ta.task_id
          AND tl.pe_reportspecid = r.pe_reportspecid); 

    INSERT INTO pe_perf_stats_report_strings
      (tstamp, 
       task_id, 
       report_name, 
       status_message)
      (SELECT sysdate,
              ta.task_id task_id,
              NVL(rs.resultname, r.NAME) NAME,
              decode(ts.status_code, sched_general.SC_Done, NULL, ts.message) message
         FROM pe_task_activity ta,
              task_status      ts,
              rg_tasklist      tl,
              pm_report        r,
              pe_reportspec    rs
        WHERE ta.task_id = pTaskId
          AND ts.task_id = ta.task_id
          AND tl.id = ta.task_id
          AND tl.pe_reportspecid = r.pe_reportspecid
          AND r.pe_reportspecid = rs.reportspec_id);

    -- First value in wins
    EXCEPTION
     WHEN DUP_VAL_ON_INDEX THEN
       NULL;

  END addReportStats;

  -- ------------------------------------------------------------------------
  --  Private helper method to decode a specific attribute
  -- ------------------------------------------------------------------------
  FUNCTION decodeAttribute(pReportAttrs   IN pe_perf_stats_report.report_attrs%TYPE,
                           pMask          IN pe_perf_stats_report.report_attrs%TYPE,
                           pAttrName      IN VARCHAR2,
                           pCurrentResult IN VARCHAR2)
    RETURN VARCHAR2
  IS
    vResult VARCHAR2(4000) := pCurrentResult;
  BEGIN

     IF (BITAND(pReportAttrs, pMask) = pMask) THEN
        IF (pCurrentResult IS NOT NULL) THEN
            vResult := pCurrentResult || ',';
        END IF;

        vResult := vResult || pAttrName;
     END IF;

     RETURN vResult;

  END decodeAttribute;

  -- ------------------------------------------------------------------------
  -- Convert the report attributes into a string
  -- Should match ReportAttributes.java
  -- ------------------------------------------------------------------------
  FUNCTION decodeReportAttrs(pReportAttrs IN pe_perf_stats_report.report_attrs%TYPE)
    RETURN VARCHAR2
  IS
    vResult VARCHAR2(4000) := NULL;
  BEGIN
    IF (pReportAttrs = 0) THEN
       RETURN NULL;
    END IF;

    vResult := decodeAttribute(pReportAttrs,   1, 'Histogram', vResult);
    vResult := decodeAttribute(pReportAttrs,   2, 'NonStoredBH', vResult);
    vResult := decodeAttribute(pReportAttrs,   4, 'StoredBH', vResult);
    vResult := decodeAttribute(pReportAttrs,   8, 'StoredSum', vResult);
    vResult := decodeAttribute(pReportAttrs,  16, 'FilterNHigh', vResult);
    vResult := decodeAttribute(pReportAttrs,  32, 'TrafficFilter', vResult);
    vResult := decodeAttribute(pReportAttrs,  64, 'EntityFilter', vResult);
    vResult := decodeAttribute(pReportAttrs, 128, 'BHFilter', vResult);
    vResult := decodeAttribute(pReportAttrs, 256, 'Ranking', vResult);
    vResult := decodeAttribute(pReportAttrs, 512, 'ActiveInstFilter', vResult);

    RETURN vResult;

  END decodeReportAttrs;

   -- ------------------------------------------------------------------------
END pe_perf_stats;
/
show errors PACKAGE BODY pe_perf_stats;
